// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "dawnopengl.hpp"
#include "game/DawnGame.hpp"
#include "display/RenderManager.hpp"
#include "display/StandardRenderPipeline.hpp"

using namespace Dawn;

RenderManager::RenderManager(DawnGame &game) :
  IRenderManager(game),
  backBuffer(*this)
{
  this->standardRenderPipeline=std::make_shared<StandardRenderPipeline>(*this);
  this->simpleShader = std::make_shared<SimpleTexturedShader>();
}

void RenderManager::init() {
  this->standardRenderPipeline->init();
  this->simpleShader->compile();
}

RenderTarget & RenderManager::getBackBuffer() {
  return this->backBuffer;
}

RenderPipeline & RenderManager::getRenderPipeline() {
  return *this->standardRenderPipeline;
}

std::shared_ptr<Shader> RenderManager::getDefaultShader() {
  return this->simpleShader;
}

void RenderManager::update() {
  this->getRenderPipeline().render();
}

RenderManager::~RenderManager() {

}