// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "DawnPokerGame.hpp"

using namespace Dawn;

DawnGame::DawnGame(DawnHost &host) :
  host(host),
  renderManager(*this)
{
}

int32_t DawnGame::init() {
  this->renderManager.init();

  this->scene = std::make_shared<Scene>(*this);

  auto cameraObject = this->scene->createSceneItem();
  auto camera = cameraObject->addComponent<Camera>();
  camera->lookAt(glm::vec3(3, 3, 3), glm::vec3(0, 0, 0));

  auto cubeObject = this->scene->createSceneItem();
  auto cubeMeshRenderer = cubeObject->addComponent<MeshRenderer>();
  auto cubeMaterial = cubeObject->addComponent<Material>();
  cubeMeshRenderer->mesh = std::make_shared<Mesh>();
  TriangleMesh::createTriangleMesh(*cubeMeshRenderer->mesh);

  return DAWN_GAME_INIT_RESULT_SUCCESS;
}

int32_t DawnGame::update(float_t delta) {
  if(this->scene != nullptr) this->scene->update();
  this->renderManager.update();
  return DAWN_GAME_UPDATE_RESULT_SUCCESS;
}

DawnGame::~DawnGame() {
  
}